;Project homepage: www.HomoFaciens.de
;Program controls a stepper motor used for an electric powered passenger car. For details visit the hompage of the project.
;Program code written for ATmega8A microcontroller
;Copyright (C) 2011 Norbert Heinz
;
;This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
;This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
;You should have received a copy of the GNU General Public License along with this program if not, see http://www.gnu.org/licenses/.

;Version 1.0

.include "m8def.inc"

.def temp01  = r16
.def temp02  = r17
.def TimeDivisor = r18         ;skip execution of timer interrupts with help of TimeDivisor
.def ControlBits = r19         ;bit 0 - adjusted forward/reverse direction
                               ;bit 1 - forward/reverse switch pressed
                               ;bit 2 - active channel for ADC
                               ;bit 3 - speed control ON
                               ;bit 4 - harware jumper timeshift (overlap)
                               ;bit 5 - braking allowed
                               ;bit 6 - detected forward/reverse direction
                               ;bit 7 - harware jumper timeshift (no overlap)

.def MotorBits = r20           ;bit 0 photo sensor 1
                               ;bit 1 photo sensor 2

.def MotorPosition = r21       ;indicator for motor movement
.def InductorSet = r22         ;which inductor is set active
.def MotorPause = r23          ;indicator for revolution speed (high value=low speed)
.def SwitchTimer = r24         ;timer forward / reverse switch
.def BrakeValue = r25          ;ADC value potentiometer brake
.def ThrottleValue = r26       ;ADC value potentiometer throttle
.def PulseCount = r27          ;Number of pulses photosensor
.def ActualSpeed = r28         ;actual rotation speed given in PulseCount
.def SetSpeed = r29            ;selected speed in PulseCount


.org 0x0000
        rjmp    main                     ;Reset Handler

.org OC1Aaddr
        rjmp    timer1_interrupt         ;Timer Overflow Handler

main:
        ;nop                             ;empty instruction to aviod upload problems concerning avrdude


        ldi     temp01, LOW(RAMEND)     ;initialize stackpointer
        out     SPL, temp01
        ldi     temp01, HIGH(RAMEND)
        out     SPH, temp01

        ;initialize ports
        ldi     temp01, 0xFF            ;Set all pins of port B as output
        out     DDRB, temp01

        ldi     temp01, 0b11111000      ;Set pins PD3-PD7 of Port D as output, PD0-PD2 as input
        out     DDRD, temp01

        ldi     temp01, 0b00000000
        out     DDRC, temp01            ;set port C as input

        ldi     temp01, 0b00111001      ;Activate pullup resistance at PC0, 3, 4, 5 (=forward/reverse switch and jumpers)
        out     PORTC, temp01

        ;initialize analog digital converter
        ldi     temp01, 1<<REFS0 | 1<<ADLAR | 1<<MUX0     ;set values for analog digital converter
        out     ADMUX, temp01           ;activate ADC

        ldi     temp01, 1<<ADEN | 1<<ADSC | 1<<ADPS2 | 1<<ADPS1
        out     ADCSRA, temp01          ;start conversion

        ;initialize timer
        ldi     temp01, high( 40000 - 1 )   ;Compare value = 40,000 pulses = 0.01 seconds until interrupt
        out     OCR1AH, temp01
        ldi     temp01, low( 40000 - 1 )
        out     OCR1AL, temp01

        ldi     temp01, ( 1 << WGM12 ) | ( 1 << CS10 )  ;turn on CTC mode, divisor 1
        out     TCCR1B, temp01

        ldi     temp01, 1 << OCIE1A  ; OCIE1A: interrupt at timer compare
        out     TIMSK, temp01

        clr     TimeDivisor
        clr     ControlBits
        clr     MotorBits
        clr     MotorPosition
        clr     InductorSet
        clr     MotorPause
        clr     PulseCount
        clr     SetSpeed
        clr     ActualSpeed

        sei     ;enable global interrupt

        ;turn magnets to initial position
        ldi     temp01, 0b10010010
        ldi     temp02, 0b01000000

        out     PORTB, temp01
        out     PORTD, temp02

wait:
        cpi     MotorPause, 255
        brlo    wait

        sbis    PIND, 1        ;NOTE: hole in disc is 1 (high), PIN 1 is bit 0, PIN 2 is bit 1!
        ori     MotorBits, 0b00000001
        sbis    PIND, 2
        ori     MotorBits, 0b00000010

        ldi     temp01, 0b00000000
        ldi     temp02, 0b00000000
        out     PORTB, temp01
        out     PORTD, temp02
        ldi     MotorPause, 0



loop:

        ;check photo sensors to detect postition of permanent magnets and inductors
        clr     temp01

        sbis    PIND, 1        ;load actual position to temp01 NOTE: hole in disc is 1 (high), PIN 1 is bit 0, PIN 2 is bit 1!
        ori     temp01, 0b00000001
        sbis    PIND, 2
        ori     temp01, 0b00000010
        clr     temp02
        sbrc    MotorBits, 0   ;load old position to temp02
        ori     temp02, 0b00000001
        sbrc    MotorBits, 1
        ori     temp02, 0b00000010
        cp      temp01, temp02
        breq    PhotoSensorEnd   ;old and actual position are equal

        cpi     PulseCount, 255  ;count sensor pulses for speed control
        breq    PulseCountSkip
        inc     PulseCount
PulseCountSkip:

        cpi     temp01, 0
        brne    PhotoSensor01
        cpi     temp02, 2
        brne    PhotoSensor01a
        inc     MotorPosition
        andi    ControlBits, 0b10111111  ;forward rotation detected
        rjmp    PhotoSensorMP
PhotoSensor01a:
        dec     MotorPosition
        ori     ControlBits, 0b01000000  ;reverse rotation detected
        rjmp    PhotoSensorMP

PhotoSensor01:
        cpi     temp01, 1
        brne    PhotoSensor02
        cpi     temp02, 0
        brne    PhotoSensor02a
        inc     MotorPosition
        andi    ControlBits, 0b10111111  ;forward rotation detected
        rjmp    PhotoSensorMP
PhotoSensor02a:
        dec     MotorPosition
        ori     ControlBits, 0b01000000  ;reverse rotation detected
        rjmp    PhotoSensorMP

PhotoSensor02:
        cpi     temp01, 3
        brne    PhotoSensor03
        cpi     temp02, 1
        brne    PhotoSensor03a
        inc     MotorPosition
        andi    ControlBits, 0b10111111  ;forward rotation detected
        rjmp    PhotoSensorMP
PhotoSensor03a:
        dec     MotorPosition
        ori     ControlBits, 0b01000000  ;reverse rotation detected
        rjmp    PhotoSensorMP

PhotoSensor03:
        cpi     temp01, 2
        brne    PhotoSensor04
        cpi     temp02, 3
        brne    PhotoSensor04a
        inc     MotorPosition
        andi    ControlBits, 0b10111111  ;forward rotation detected
        rjmp    PhotoSensorMP
PhotoSensor04a:
        dec     MotorPosition
        ori     ControlBits, 0b01000000  ;reverse rotation detected
        rjmp    PhotoSensorMP

PhotoSensor04:

PhotoSensorMP:           ;check if MotorPosition is out of range
        cpi     MotorPosition, 6
        brne    PhotoSensorMP01
        clr     MotorPosition
PhotoSensorMP01:
        cpi     MotorPosition, 255
        brne    PhotoSensorMP02
        ldi     MotorPosition, 5
PhotoSensorMP02:
        ;load new sensor position to MotorBits
        andi    MotorBits, 0b11111100   ;clear
        sbrc    temp01, 0
        ori     MotorBits, 0b00000001   ;load new
        sbrc    temp01, 1
        ori     MotorBits, 0b00000010

        clr     MotorPause              ;motor has moved, so clear timer

PhotoSensorEnd:

        ;set active inductor
        clr     InductorSet
        sbrc    ControlBits, 0
        rjmp    InductorSetReverse

        cpi     MotorPosition, 1
        brsh    InductorSet01
        ldi     InductorSet, 0b10000001    ;bits 0-2 = inductors used to accelerate, bits 5-7 = inductors set for braking
        rjmp    InductorSetEnd

InductorSet01:
        cpi     MotorPosition, 2
        brsh    InductorSet02
        sbrs    ControlBits, 7           ;time shift without overlap set
        ldi     InductorSet, 0b01000001
        sbrc    ControlBits, 7           ;time shift without overlap disabled
        ldi     InductorSet, 0b01000100
        sbrc    ControlBits, 4           ;time shift with overlap disabled
        ldi     InductorSet, 0b01000101
        rjmp    InductorSetEnd

InductorSet02:
        cpi     MotorPosition, 3
        brsh    InductorSet03
        ldi     InductorSet, 0b01000100
        rjmp    InductorSetEnd

InductorSet03:
        cpi     MotorPosition, 4
        brsh    InductorSet04
        sbrs    ControlBits, 7           ;time shift without overlap set
        ldi     InductorSet, 0b00100100
        sbrc    ControlBits, 7           ;time shift without overlap disabled
        ldi     InductorSet, 0b00100010
        sbrc    ControlBits, 4           ;time shift with overlap disabled
        ldi     InductorSet, 0b00100110
        rjmp    InductorSetEnd

InductorSet04:
        cpi     MotorPosition, 5
        brsh    InductorSet05
        ldi     InductorSet, 0b00100010
        rjmp    InductorSetEnd

InductorSet05:
        sbrs    ControlBits, 7           ;time shift without overlap set
        ldi     InductorSet, 0b10000010
        sbrc    ControlBits, 7           ;time shift without overlap disabled
        ldi     InductorSet, 0b10000001
        sbrc    ControlBits, 4           ;time shift with overlap disabled
        ldi     InductorSet, 0b10000011
        rjmp    InductorSetEnd

InductorSetReverse:


        cpi     MotorPosition, 1
        brsh    InductorSet07
        ldi     InductorSet, 0b00100100
        rjmp    InductorSetEnd

InductorSet07:
        cpi     MotorPosition, 2
        brsh    InductorSet08
        sbrs    ControlBits, 7           ;time shift without overlap set
        ldi     InductorSet, 0b00100010
        sbrc    ControlBits, 7           ;time shift without overlap disabled
        ldi     InductorSet, 0b00100100
        sbrc    ControlBits, 4           ;time shift with overlap disabled
        ldi     InductorSet, 0b00100110
        rjmp    InductorSetEnd

InductorSet08:
        cpi     MotorPosition, 3
        brsh    InductorSet09
        ldi     InductorSet, 0b10000010
        rjmp    InductorSetEnd

InductorSet09:
        cpi     MotorPosition, 4
        brsh    InductorSet10
        sbrs    ControlBits, 7           ;time shift without overlap set
        ldi     InductorSet, 0b10000001
        sbrc    ControlBits, 7           ;time shift without overlap disabled
        ldi     InductorSet, 0b10000010
        sbrc    ControlBits, 4           ;time shift with overlap disabled
        ldi     InductorSet, 0b10000011
        rjmp    InductorSetEnd

InductorSet10:
        cpi     MotorPosition, 5
        brsh    InductorSet11
        ldi     InductorSet, 0b01000001
        rjmp    InductorSetEnd

InductorSet11:

        sbrs    ControlBits, 7           ;time shift without overlap set
        ldi     InductorSet, 0b01000100
        sbrc    ControlBits, 7           ;time shift without overlap disabled
        ldi     InductorSet, 0b01000001
        sbrc    ControlBits, 4           ;time shift with overlap disabled
        ldi     InductorSet, 0b01000101

InductorSetEnd:

        ;disable braking if detected rotation differs form adjusted rotation
        cpi     BrakeValue, 0
        brne    BrakeAllow01
        ori     ControlBits, 0b00100000  ;allow braking

BrakeAllow01:
        sbrs    ControlBits, 0
        rjmp    BrakeAllow02
        sbrc    ControlBits, 6
        rjmp    BrakeAllow02
        andi    ControlBits, 0b11011111  ;directions differ, so forbid braking

BrakeAllow02:
        sbrc    ControlBits, 0
        rjmp    BrakeAllow03
        sbrs    ControlBits, 6
        rjmp    BrakeAllow03
        andi    ControlBits, 0b11011111  ;directions differ, so forbid braking

BrakeAllow03:
        ;forbid braking if motor has already stopped
        cpi     MotorPause, 100          ;no pulses for 100 timer interrupts (=1second)
        brlo    BrakeAllowEnd            ;if motor is still rotating
        andi    ControlBits, 0b11011111  ;breaking not necessary, because motor has already stopped

BrakeAllowEnd:

        ;check hardware jumper status
        sbrs    controlbits, 3
        clr     SetSpeed              ;clear value SetSpeed if speedcontrol was disabled at last pass
        mov     temp01, ControlBits
        andi    temp01, 0b01100111
        sbis    PINC, 5               ;skip next operation, if bit 5 is set (1, high, jumper speed control open)
        ori     temp01, 0b00001000
        sbis    PINC, 3               ;skip next operation, if bit 3 is set (1, high, jumper timing overlap open)
        ori     temp01, 0b00010000
        sbis    PINC, 4               ;skip next operation, if bit 4 is set (1, high, jumper timing open)
        ori     temp01, 0b10000000
        mov     ControlBits, temp01


        ;check throttle / breake
        sbic    ADCSRA, ADSC
        rjmp    SkipThrottle
        in      temp01, ADCL        ;read LOW byte
        in      temp02, ADCH        ;read HIGH byte

        ldi     temp01, 0b00000100
        eor     ControlBits, temp01        ;toggle bit 2 (indicator for ADC channel)

        sbrs    ControlBits, 2
        ldi     temp01, 1<<REFS0 | 1<<ADLAR | 1<<MUX0     ;set values for analog digital converter channel 1 (brake)
        sbrc    ControlBits, 2
        ldi     temp01, 1<<REFS0 | 1<<ADLAR | 1<<MUX1     ;set values for analog digital converter channel 2 (throttle)
        out     ADMUX, temp01           ;activate ADC

        sbi     ADCSRA, ADSC            ;start next conversion


        ldi     temp01, 20
        cpi     temp02, 50
        brlo    SetThrottle

        ldi     temp01, 15
        cpi     temp02, 100
        brlo    SetThrottle

        ldi     temp01, 10
        cpi     temp02, 150
        brlo    SetThrottle

        ldi     temp01, 5
        cpi     temp02, 200
        brlo    SetThrottle

        ldi     temp01, 0


SetThrottle:
        sbrc    ControlBits, 2
        mov     BrakeValue, temp01        ;skipped if bit 3 of ControlBits is 0 (ADC indicated ThrottleValue)
        sbrs    ControlBits, 2
        mov     ThrottleValue, temp01     ;skipped if bit 3 of ControlBits is 1 (ADC indicated BrakeValue)

SkipThrottle:

        ;check forward/reverse switch
        cpi     MotorPause, 100           ;no pulses for 100 timer interrupts ( = 1 second)
        brlo    EndFR                     ;switch can only be actuated if motor is not running
        sbic    PINC, 0                   ;skip next operation, if bit 0 is cleared (0, low, forward/reverse switch pressed)
        rjmp    ContinueFR01
        ;forward/reverse switch pressed
        sbrs    ControlBits, 1            ;skip next operation if switch was already pressed during last pass
        ldi     SwitchTimer, 100          ;initialize SwitchTimer (switch must be pressed for 100 timer interrupts = 1 second)
        ori     ControlBits, 0b00000010   ;set bit number 1 to 1 (high), leaving other bits unchanged (switch is pressed)
        rjmp    EndFR

ContinueFR01:
        ;forward/reverse switch released
        cpi     SwitchTimer, 0
        brne    ContinueFR02              ;branch if switch wasn't pressed long enough
        sbrs    ControlBits, 1
        rjmp    EndFR                     ;no jump if switch was pressed during previous pass
        ldi     temp01, 0b00000001
        eor     ControlBits, temp01       ;toggle bit 0 (indicator for motor direction)
        sbrc    ControlBits, 0
        sbi     PORTD, 3                  ;set bit number 3 of Port D (control light forward/reverse switch)
        sbrs    ControlBits, 0
        cbi     PORTD, 3                  ;clear bit number 3 of Port D (control light forward/reverse switch)
        clr     SetSpeed                  ;clear speed control value to avoid unexpected behavior

ContinueFR02:
        andi     ControlBits, 0b11111101  ;set bit number 1 to 0 (low), switch not pressed
        ldi      SwitchTimer, 0

EndFR:

        ;step motor
        clr     temp01
        clr     temp02

        cpi     ThrottleValue, 0
        breq    NoSpeedSet01
        mov     SetSpeed, ActualSpeed

NoSpeedSet01:

        cpi     ThrottleValue, 5
        brne    ContinueStep01
        rcall   Sub_StepMotor01
ContinueStep01:
        cpi     ThrottleValue, 10
        brne    ContinueStep02
        rcall   Sub_StepMotor02
ContinueStep02:
        cpi     ThrottleValue, 15
        brne    ContinueStep05
        rcall   Sub_StepMotor03
ContinueStep05:
        cpi     ThrottleValue, 20
        brne    ContinueStep06
        rcall   Sub_StepMotor04
ContinueStep06:

        cpi     ThrottleValue, 0     ;no throttle set
        brne    ContinueStep10

        cpi     BrakeValue, 0
        breq    NoSpeedSet02
        mov     SetSpeed, ActualSpeed

NoSpeedSet02:

        cpi     BrakeValue, 0
        breq    ContinueStep10
        sbrs    ControlBits, 5       ;is braking allowed?
        rjmp    ContinueStep10

        cpi     BrakeValue, 5
        brne    ContinueStep07
        rcall   Sub_BrakeMotor01
ContinueStep07:
        cpi     BrakeValue, 10
        brne    ContinueStep08
        rcall   Sub_BrakeMotor02
ContinueStep08:
        cpi     BrakeValue, 15
        brne    ContinueStep09
        rcall   Sub_BrakeMotor03
ContinueStep09:
        cpi     BrakeValue, 20
        brne    ContinueStep10
        rcall   Sub_BrakeMotor04
ContinueStep10:

        sbrs    ControlBits, 3
        rjmp    SpeedControlEnd
        cpi     ThrottleValue, 0
        brne    SpeedControlEnd
        cpi     BrakeValue, 0
        brne    SpeedControlEnd
        ;compare ActualSpeed and SetSpeed
        ;the control loop is influenced by the time intervalls of speed measurement
        ;and the 'throttle' set per pulse difference
        cp      ActualSpeed, SetSpeed
        brsh    SpeedControlEnd
        mov     temp01, SetSpeed
        sub     temp01, ActualSpeed

        cpi     temp01, 2         ;pulse differende 2 pulses
        brsh    SpeedControl01
        rcall   Sub_StepMotor01
        rjmp    SpeedControlEnd

SpeedControl01:
        cpi     temp01, 3         ;pulse differende 3 pulses
        brsh    SpeedControl02
        rcall   Sub_StepMotor02
        rjmp    SpeedControlEnd

SpeedControl02:
        cpi     temp01, 4         ;pulse differende 4 pulses
        brsh    SpeedControl03
        rcall   Sub_StepMotor03
        rjmp    SpeedControlEnd

SpeedControl03:
        rcall   Sub_StepMotor04

SpeedControlEnd:


        rcall   Sub_SetPorts
        rjmp    loop


;++++++++++++++++++++++++++++++++++ Motor step 01 +++++++++++++++++++++++++++++++++++++++++

Sub_StepMotor01:

        clr     temp01
        clr     temp02

        sbrc    InductorSet, 0
        ori     temp01, 0b00000001

        sbrc    InductorSet, 1
        ori     temp01, 0b00000010

        sbrc    InductorSet, 2
        ori     temp01, 0b00000100


        ret
;---------------------------------- End motor step 01 -------------------------------------

;++++++++++++++++++++++++++++++++++ Motor step 02 +++++++++++++++++++++++++++++++++++++++++

Sub_StepMotor02:

        clr     temp01
        clr     temp02

        sbrc    InductorSet, 0
        ori     temp01, 0b01000001

        sbrc    InductorSet, 1
        ori     temp01, 0b10000010

        sbrc    InductorSet, 2
        ori     temp01, 0b00000100
        sbrc    InductorSet, 2
        ori     temp02, 0b00010000


        ret
;---------------------------------- End motor step 02 -------------------------------------

;++++++++++++++++++++++++++++++++++ Motor step 03 +++++++++++++++++++++++++++++++++++++++++

Sub_StepMotor03:

        clr     temp01
        clr     temp02

        sbrc    InductorSet, 0
        ori     temp01, 0b01001001

        sbrc    InductorSet, 1
        ori     temp01, 0b10010010

        sbrc    InductorSet, 2
        ori     temp01, 0b00100100
        sbrc    InductorSet, 2
        ori     temp02, 0b00010000


        ret
;---------------------------------- End motor step 03 -------------------------------------


;++++++++++++++++++++++++++++++++++ Motor step 04 +++++++++++++++++++++++++++++++++++++++++

Sub_StepMotor04:

        clr     temp01
        clr     temp02


        sbrc    InductorSet, 0
        ori     temp01, 0b01001001
        sbrc    InductorSet, 0
        ori     temp02, 0b00100000

        sbrc    InductorSet, 1
        ori     temp01, 0b10010010
        sbrc    InductorSet, 1
        ori     temp02, 0b01000000

        sbrc    InductorSet, 2
        ori     temp01, 0b00100100
        sbrc    InductorSet, 2
        ori     temp02, 0b10010000


        ret
;---------------------------------- End motor step 04 -------------------------------------



;++++++++++++++++++++++++++++++++++ Brake motor 01 +++++++++++++++++++++++++++++++++++++++++

Sub_BrakeMotor01:

        clr     temp01
        clr     temp02

        sbrc    InductorSet, 5
        ori     temp01, 0b00000001

        sbrc    InductorSet, 6
        ori     temp01, 0b00000010

        sbrc    InductorSet, 7
        ori     temp01, 0b00000100


        ret
;---------------------------------- End brake motor 01 ------------------------------------

;++++++++++++++++++++++++++++++++++ Brake motor 02 ++++++++++++++++++++++++++++++++++++++++

Sub_BrakeMotor02:

        clr     temp01
        clr     temp02

        sbrc    InductorSet, 5
        ori     temp01, 0b01000001

        sbrc    InductorSet, 6
        ori     temp01, 0b10000010

        sbrc    InductorSet, 7
        ori     temp01, 0b00000100
        sbrc    InductorSet, 7
        ori     temp02, 0b00010000


        ret
;---------------------------------- End brake motor 02 ------------------------------------

;++++++++++++++++++++++++++++++++++ Brake motor 03 ++++++++++++++++++++++++++++++++++++++++

Sub_BrakeMotor03:

        clr     temp01
        clr     temp02

        sbrc    InductorSet, 5
        ori     temp01, 0b01001001

        sbrc    InductorSet, 6
        ori     temp01, 0b10010010

        sbrc    InductorSet, 7
        ori     temp01, 0b00100100
        sbrc    InductorSet, 7
        ori     temp02, 0b00010000


        ret
;---------------------------------- End brake motor 03 -------------------------------------


;++++++++++++++++++++++++++++++++++ Brake motor 04 +++++++++++++++++++++++++++++++++++++++++

Sub_BrakeMotor04:

        clr     temp01
        clr     temp02

        sbrc    InductorSet, 5
        ori     temp01, 0b01001001
        sbrc    InductorSet, 5
        ori     temp02, 0b00100000

        sbrc    InductorSet, 6
        ori     temp01, 0b10010010
        sbrc    InductorSet, 6
        ori     temp02, 0b01000000

        sbrc    InductorSet, 7
        ori     temp01, 0b00100100
        sbrc    InductorSet, 7
        ori     temp02, 0b10010000


        ret
;---------------------------------- End brake motor 04 -------------------------------------




;++++++++++++++++++++++++++++++++++ set ports +++++++++++++++++++++++++++++++++++++++++++++
Sub_SetPorts:                               ;set output pins for motor rotation

        sbrc    ControlBits, 0              ;skip next operation if bit 0 is cleared (forward/reverse indicator)
        ori     temp02, 0b00001000          ;set red LED to ON

        out     PORTB, temp01
        out     PORTD, temp02

        ret

;---------------------------------- End set ports -----------------------------------------

;++++++++++++++++++++++++++++++++++ Timer interrupt +++++++++++++++++++++++++++++++++++++++

timer1_interrupt:                     ;Timer 1 interrupt handler
        ;100 interrupts during 1 second when running at 4MHz
        push    temp01                ;save temp01
        in      temp01, SREG          ;save status register
        push    temp01

        cpi     SwitchTimer, 0
        breq    TimerContinue01
        dec     SwitchTimer

TimerContinue01:
        cpi     MotorPause, 255
        breq    TimerContinue02
        inc     MotorPause

TimerContinue02:

        inc     TimeDivisor
        cpi     TimeDivisor, 4
        brlo    TimerContinue03
        ;24 pulses per revolution
        mov     ActualSpeed, PulseCount
        clr     PulseCount
        clr     TimeDivisor

TimerContinue03:

        pop     temp01
        out     SREG, temp01          ;restore status register
        pop     temp01                ;restore temp01

        reti